import { decrypt, encrypt } from 'romulus-js'
import { DEFAULT_KEY, MessageTypes } from '../common'
import { numberToUint16BE } from '../utilities/number'
import { packOutgoingPacket } from './packet'

const MESSAGE_TYPE = numberToUint16BE(MessageTypes.Basic)

export interface BasicMessage {
  message: string
  success?: boolean
}

/**
 * Create an outgoing basic message (0x0001) packet.
 * @param properties The properties for the message.
 * @param key The key to encrypt the data with.
 * @returns An outgoing basic message (0x0001) packet.
 */
export function packBasicMessage (properties: BasicMessage, key: Uint8Array = DEFAULT_KEY): Uint8Array {
  const encoder = new TextEncoder()
  const message = encoder.encode(properties.message)
  const data = encrypt(message, MESSAGE_TYPE, key)
  return packOutgoingPacket({
    messageType: MESSAGE_TYPE,
    data: data
  })
}

/**
 * Unpack the data section of an incoming basic message (0x0001) message.
 * @param data The data section of an incoming basic message (0x0001) message.
 * @param key The key to decrypt the data with.
 * @returns An unpacked basic message (0x0001) message.
 */
export function unpackBasicMessage (data: Uint8Array, key: Uint8Array = DEFAULT_KEY): BasicMessage {
  const decoder = new TextDecoder()
  const message = decrypt(data, MESSAGE_TYPE, key)
  return {
    message: decoder.decode(message.plaintext),
    success: message.success
  }
}
